
#include <EEPROM.h>
#include <MsTimer2.h>
#include <Encoder.h>
#include <TM1637Display.h>

//#define Fuse            2                                 // Вход прерывания. Датчик Фазы
#define AC_LOAD           6                                 // Вывод для управления симистором
/*-----------------------------------------------------------------------------------------------------*/
Encoder myEnc(3, 5);
#define SB_Enc            4
long oldPosition        = -999;
#define KEY_Start         12
byte SB_Touch           = 0;
/*-----------------------------------------------------------------------------------------------------*/
#define CLK   11
#define DIO   10 

TM1637Display display(CLK, DIO);      // Set up the 4-Digit Display.

uint8_t data[]  = {0, 0, 0, 0};

int  Digit_0[]  = {0xBF,0x86,0xDB,0xCF,0xE6,0xED,0xAD,0x87,0xFF,0xEF};
int  Digit_1[]  = {0x00,0x40,0xCF,0xCF,0xF3,0xB9,0xEE,0xEE,0xBF,0x00};
int  Digit_2[]  = {0x00,0x06,0x5B,0x4F,0x66,0x6D,0x7D,0x07,0x7F,0x6F};
int  Digit_3[]  = {0x3F,0x06,0x5B,0x4F,0x66,0x6D,0x7D,0x07,0x7F,0x6F};
/*-----------------------------------------------------------------------------------------------------*/
int  PWM[]                = {0, 0};
byte Fuse_Pause_ON        = 5;

unsigned long time;
unsigned long timenext    = 0;

byte Start_ON             = 0;
byte digitBuffer[]        = {5, 1, 0, 5, 0};

byte Impuls               = 1;
byte Select               = 1;
byte Second_Impuls        = 0;
int  Sec[]                = {0, 0, 0};
byte Write                = 0;

#define Led_ON   digitBuffer[2] = 1; Display(); for (byte i = 0; i <= 3; i++) display.setSegments(data + i, 1, i);
#define Led_OFF  digitBuffer[2] = 0; Display(); for (byte i = 0; i <= 3; i++) display.setSegments(data + i, 1, i);
/*=======================================================================================================
                                              Чтение EEPROM
=======================================================================================================*/
void EEPROM_Read()
{
  for (int i = 0; i <= 4; i++)
  {
    digitBuffer[i] = EEPROM.read(i);                // считываем значение по текущему адресу EEPROM
  }
  Fuse_Pause_ON = digitBuffer[0];
  PWM[0] = (digitBuffer[1] * 1000) + (digitBuffer[2] * 100) + (digitBuffer[3] * 10) + digitBuffer[4];
  PWM[1] = (digitBuffer[2] * 100) + (digitBuffer[3] * 10) + digitBuffer[4];
  Impuls = digitBuffer[1];
}
/*=======================================================================================================
                                              Запись EEPROM
=======================================================================================================*/
void EEPROM_Write()
{
  for (int i = 0; i <= 4; i++)
  {
    EEPROM.write(i, digitBuffer[i]);
  }
}
/*=======================================================================================================
                                    Функция включения симистора по прерыванию                               
=======================================================================================================*/
void Front_Up()                                       // Прерывание по изменению уровня с низкого на высокий
{
  delay(Fuse_Pause_ON);                               // Пауза для включения симистора на пике волны
  digitalWrite(AC_LOAD, HIGH);                        // Включить симистор
  detachInterrupt(0);     
}
/*=======================================================================================================
                                              Инициализация
=======================================================================================================*/
void setup()
{
  Serial.begin(115200);                               // Скорость вывода в консоль

  display.setBrightness(0x0F);                        // Устанавливаем яркость дисплея
  display.showNumberDec(1234);

  pinMode(AC_LOAD, OUTPUT);                           // Вывод назначаем как выход
  digitalWrite(AC_LOAD, LOW);
  delay(500);
  pinMode(SB_Enc, INPUT_PULLUP);
  pinMode(KEY_Start, INPUT_PULLUP);
    
  Serial.println(" << WELDING >> ");
  
//  EEPROM_Write();
  EEPROM_Read();
  MsTimer2::set(50, Timing_ISR);                        // Задаем период прерывания по таймеру 50 мс
  MsTimer2::start();                                    // Разрешаем прерывание по таймеру
}
/*=======================================================================================================
                                              Главный цикл
=======================================================================================================*/
void loop()
{
  Display();

  time = millis();  
  if (time >= timenext)
  {
    timenext += PWM[1];                               // Длительность импульса
    digitalWrite(AC_LOAD, LOW);                       // Выключение симистора(он выключится когда полупериод достигнет нуля)
    digitBuffer[2] = 0;
    if (Second_Impuls == 0) {Sec[0] = 0; Second_Impuls = 1;}
  }
}
/*=======================================================================================================
                                              Encoder
=======================================================================================================*/
void Encode()
{
  long newPosition = myEnc.read() / 3;
  if (oldPosition != newPosition) 
  {
    if (oldPosition < newPosition)
    {
      if (Select == 0) if (Fuse_Pause_ON <= 4) Fuse_Pause_ON++; digitBuffer[0] = Fuse_Pause_ON;
      if (Select == 2) if (PWM[1] <= 98) PWM[1]++;
      digitBuffer[3] = PWM[1] / 10; int Temp = digitBuffer[3] * 10; digitBuffer[4] = PWM[1] - Temp;
      if (Select == 3) if (Impuls == 1) Impuls += 1; digitBuffer[1] = Impuls;
    }
    if (oldPosition > newPosition)
    {
      if (Select == 0) if (Fuse_Pause_ON >= 2) Fuse_Pause_ON--; digitBuffer[0] = Fuse_Pause_ON;
      if (Select == 2) if (PWM[1] >= 2) PWM[1]--;
      digitBuffer[3] = PWM[1] / 10; int Temp = digitBuffer[3] * 10; digitBuffer[4] = PWM[1] - Temp;
      if (Select == 3) if (Impuls == 2) Impuls -= 1; digitBuffer[1] = Impuls;
    }
    oldPosition = newPosition;
  }
/*-----------------------------------------------------------------------------------------------------*/
    if (digitalRead(SB_Enc) == 0)
    {
      switch (Select)
      {
        case 0: Select = 1; break;
        case 1: Select = 2; break;
        case 2: Select = 3; break;
        case 3: Select = 0; break;
      }
      SB_Touch = 0; Sec[2] = 0;
      while (SB_Touch == 0)                                 // Ждем пока не отпустят кнопку
      {
        if (digitalRead(SB_Enc) == 0) SB_Touch = 0; else SB_Touch = 1;
        Sec[2]++;
        if (Sec[2] >= 300)
        {
          Sec[1] = 0; Sec[2] = 0;
          Write = 1; EEPROM_Write(); EEPROM_Read();
          Led_ON  
          Select = 1; SB_Touch = 1;
        }
        delay(10);        
      }
    }
}
/*=======================================================================================================
                                              Динамическая индикация
=======================================================================================================*/
void Display()
{
  for (byte i = 0; i <= 9; i++)                                 // Загружаем значения цифр от 0 до 9
  {
    if (digitBuffer[1] == i) data[0] = Digit_3[i];
    if (digitBuffer[2] == i) data[1] = Digit_1[i];
    if (digitBuffer[3] == i) data[2] = Digit_2[i];
    if (digitBuffer[4] == i) data[3] = Digit_3[i];
  }
}
/*=====================================================================================================*/
